@@ -38,6 +38,7 @@ from hexview import HexView
 from backend import GameConquerorBackend
 import misc
 
+CLIPBOARD = gtk.clipboard_get()
 WORK_DIR = os.path.dirname(sys.argv[0])
 DATA_WORKER_INTERVAL = 500 # for read(update)/write(lock)
 SCAN_RESULT_LIST_LIMIT = 1000 # maximal number of entries that can be displayed
@@ -108,6 +109,7 @@ TYPESIZES = {'int8':1
 
 class GameConqueror():
     def __init__(self):
+        gtk.settings_get_default().set_long_property('gtk-tooltip-timeout', 0, '')
         ##################################
         # init GUI
         self.builder = gtk.Builder()
@@ -161,7 +163,7 @@ class GameConqueror():
         # we may need a cell data func here
         # create model
         self.scanresult_tv = self.builder.get_object('ScanResult_TreeView')
-        self.scanresult_liststore = gtk.ListStore(str, str, str) 
+        self.scanresult_liststore = gtk.ListStore(str, str, str, bool) #addr, value, type, valid
         self.scanresult_tv.set_model(self.scanresult_liststore)
         # init columns
         misc.treeview_append_column(self.scanresult_tv, 'Address', attributes=(('text',0),), properties = (('family', 'monospace'),))
@@ -169,10 +171,12 @@ class GameConqueror():
 
         # init CheatList TreeView
         self.cheatlist_tv = self.builder.get_object('CheatList_TreeView')
-        self.cheatlist_liststore = gtk.ListStore(str, bool, str, str, str, str)
+        self.cheatlist_liststore = gtk.ListStore(str, bool, str, str, str, str, bool) #lockflag, locked, description, addr, type, value, valid
         self.cheatlist_tv.set_model(self.cheatlist_liststore)
+        self.cheatlist_tv.set_reorderable(True)
         self.cheatlist_updates = []
         self.cheatlist_editing = False
+        self.cheatlist_tv.connect('key-press-event', self.cheatlist_keypressed)
         # Lock Flag
         misc.treeview_append_column(self.cheatlist_tv, '' 
                                         ,renderer_class = gtk.CellRendererCombo
@@ -230,11 +234,16 @@ class GameConqueror():
                                                     ('editing-canceled', self.cheatlist_edit_cancel),)
                                    )
 
+        # init ProcessList
+        self.processfilter_input = self.builder.get_object('ProcessFilter_Input')
+        self.userfilter_input = self.builder.get_object('UserFilter_Input')
         # init ProcessList_TreeView
         self.processlist_tv = self.builder.get_object('ProcessList_TreeView')
         self.processlist_tv.get_selection().set_mode(gtk.SELECTION_SINGLE)
-        self.processlist_liststore = gtk.ListStore(str, str)
-        self.processlist_tv.set_model(self.processlist_liststore)
+        self.processlist_liststore = gtk.ListStore(str, str, str)
+        self.processlist_filter = self.processlist_liststore.filter_new(root=None)
+        self.processlist_filter.set_visible_func(self.processlist_filter_func, data=None)
+        self.processlist_tv.set_model(self.processlist_filter)
         self.processlist_tv.set_enable_search(True)
         self.processlist_tv.set_search_column(1)
         # first col
@@ -242,10 +251,14 @@ class GameConqueror():
                                         ,attributes = (('text',0),)
                                    )
         # second col
-        misc.treeview_append_column(self.processlist_tv, 'Process'
+        misc.treeview_append_column(self.processlist_tv, 'User'
                                         ,attributes = (('text',1),)
                                    )
 
+        # third col
+        misc.treeview_append_column(self.processlist_tv, 'Process'
+                                        ,attributes = (('text',2),)
+                                   )
         # init AddCheatDialog
         self.addcheat_address_input = self.builder.get_object('Address_Input')
         self.addcheat_description_input = self.builder.get_object('Description_Input')
@@ -277,6 +290,10 @@ class GameConqueror():
                              ,'Browse this address'
                              ,self.cheatlist_popup_cb
                              ,'browse_this_address')
+        misc.menu_append_item(self.cheatlist_popup
+                             ,'Copy address'
+                             ,self.cheatlist_popup_cb
+                             ,'copy_address')
         misc.menu_append_item(self.cheatlist_popup
                              ,'Remove this entry'
                              ,self.cheatlist_popup_cb
@@ -293,6 +310,7 @@ class GameConqueror():
         self.last_hexedit_address = (0,0) # used for hexview
         self.is_scanning = False
         self.exit_flag = False # currently for data_worker only, other 'threads' may also use this flag
+        self.is_data_worker_working = False
 
         self.backend = GameConquerorBackend()
         self.backend.add_error_listener(self.backend_error_cb)
@@ -393,10 +411,16 @@ class GameConqueror():
             return True
         return False
 
+    def ProcessFilter_Input_changed_cb(self, widget, data=None):
+        self.processlist_filter.refilter()
+
+    def UserFilter_Input_changed_cb(self, widget, data=None):
+        self.processlist_filter.refilter()
+
     def ProcessList_TreeView_row_activated_cb(self, treeview, path, view_column, data=None):
         (model, iter) = self.processlist_tv.get_selection().get_selected()
         if iter is not None:
-            (pid, process) = model.get(iter, 0, 1)
+            (pid, user, process) = model.get(iter, 0, 1, 2)
             self.select_process(int(pid), process)
             self.process_list_dialog.response(gtk.RESPONSE_CANCEL)
             return True
@@ -406,7 +430,8 @@ class GameConqueror():
         self.processlist_liststore.clear()
         pl = self.get_process_list()
         for p in pl:
-            self.processlist_liststore.append([p[0], p[1][:50]]) # limit the length here, otherwise it may crash (why?)
+           # self.processlist_liststore.append([p[0], (p[1][:50] if len(p) > 1 else '<unknown>')]) # limit the length here, otherwise it may crash (why?)
+            self.processlist_liststore.append([p[0], (p[1] if len(p) > 1 else '<unknown>'),(p[2] if len(p) > 2 else '<unknown>')])
         self.process_list_dialog.show()
         while True:
             res = self.process_list_dialog.run()
@@ -491,6 +516,14 @@ class GameConqueror():
             self.scan_for_addr(int(addr,16))
             return True
         return False
+        
+    def cheatlist_keypressed(self, cheatlist_tv, event, selection=None):
+        keycode = event.keyval
+        pressedkey = gtk.gdk.keyval_name(keycode)
+        if pressedkey == 'Delete':
+            (model, iter) = self.cheatlist_tv.get_selection().get_selected()
+            if iter is None: return
+            self.cheatlist_liststore.remove(iter) 
 
     def cheatlist_popup_cb(self, menuitem, data=None):
         self.cheatlist_editing = False
@@ -504,13 +537,17 @@ class GameConqueror():
         elif data == 'browse_this_address':
             self.browse_memory(int(addr,16))
             return True
+        elif data == 'copy_address':
+            CLIPBOARD.set_text(addr)
+            return True
         return False
 
     def cheatlist_toggle_lock_cb(self, cellrenderertoggle, path, data=None):
         row = int(path)
-        locked = self.cheatlist_liststore[row][1]
-        locked = not locked
-        self.cheatlist_liststore[row][1] = locked
+        if self.cheatlist_liststore[row][6]: # valid
+            locked = self.cheatlist_liststore[row][1]
+            locked = not locked
+            self.cheatlist_liststore[row][1] = locked
         if locked:
             #TODO: check value(valid number & not overflow), if failed, unlock it and do nothing
             pass
@@ -540,13 +577,15 @@ class GameConqueror():
         if new_text == '':
             return True
         row = int(path)
+        if not self.cheatlist_liststore[row][6]: #not valid
+            return True
         self.cheatlist_liststore[row][5] = new_text
         if self.cheatlist_liststore[row][1]: # locked
             # data_worker will handle this
             pass
         else:
             # write it for once
-            (lockflag, locked, desc, addr, typestr, value) = self.cheatlist_liststore[row]
+            (lockflag, locked, desc, addr, typestr, value, valid) = self.cheatlist_liststore[row]
             self.cheatlist_updates.append(row)
             self.write_value(addr, typestr, value)
         return True
@@ -561,6 +600,15 @@ class GameConqueror():
             pass
         return True
 
+    def processlist_filter_func(self, model, iter, data=None):
+        (pid, user, process) = model.get(iter, 0, 1, 2)
+        return process is not None and \
+                process.find(self.processfilter_input.get_text()) != -1 and \
+                user is not None and \
+                user.find(self.userfilter_input.get_text()) != -1
+        
+
+
     ############################
     # core functions
     def show_error(self, msg):
@@ -598,6 +646,8 @@ class GameConqueror():
 
     # parse bytes dumped by scanmem into number, string, etc.
     def bytes2value(self, typename, bytes):
+        if bytes is None:
+            return None
         if typename in TYPENAMES_G2STRUCT.keys():
             return struct.unpack(TYPENAMES_G2STRUCT[typename], bytes)[0]
         elif typename == 'string':
@@ -648,13 +698,12 @@ class GameConqueror():
             addr = selected_region['start_addr']
 
         # read region if necessary
-        start_addr = selected_region['start_addr']
-        end_addr = selected_region['end_addr']
+        start_addr = max(addr - 1024, selected_region['start_addr'])
+        end_addr = min(addr + 1024, selected_region['end_addr'])
         if self.last_hexedit_address[0] != start_addr or \
            self.last_hexedit_address[1] != end_addr:
-            try:
-                data = self.read_memory(start_addr, end_addr - start_addr)
-            except IOError,e:
+            data = self.read_memory(start_addr, end_addr - start_addr)
+            if data is None:
                 self.show_error('Cannot read memory')
                 return
             self.last_hexedit_address = (start_addr, end_addr)
@@ -671,7 +720,9 @@ class GameConqueror():
     # this callback will be called from other thread
     def backend_error_cb(self, msg):
         gtk.gdk.threads_enter()
-        self.show_error('Backend error: %s'%(msg,))
+        # we don't want massive error output for data worker
+        if not self.is_data_worker_working:
+            self.show_error('Backend error: %s'%(msg,))
         if self.is_scanning:
             self.finish_scan()
         gtk.gdk.threads_leave()
@@ -695,10 +746,10 @@ class GameConqueror():
             if TYPENAMES_S2G.has_key(t):
                 vt = TYPENAMES_S2G[t]
                 break
-        self.cheatlist_liststore.prepend(['=', False, description, addr, vt, value])
+        self.cheatlist_liststore.prepend(['=', False, description, addr, vt, value, True])
 
     def get_process_list(self):
-        return [e.strip().split(' ',1) for e in os.popen('ps -wweo pid=,command= --sort=-pid').readlines()]
+        return [map(str.strip, e.strip().split(' ',2)) for e in os.popen('ps -wweo pid=,user=,command= --sort=-pid').readlines()]
 
     def select_process(self, pid, process_name):
         # ask backend for attaching the target process
@@ -832,7 +883,7 @@ class GameConqueror():
                 (a, v, t) = map(str.strip, line.split(',')[:3])
                 a = '%x'%(int(a,16),)
                 t = t[1:-1]
-                self.scanresult_liststore.append([a, v, t])
+                self.scanresult_liststore.append([a, v, t, True])
             self.scanresult_tv.set_model(self.scanresult_liststore)
 
     # return (r1, r2) where all rows between r1 and r2 (INCLUSIVE) are visible
@@ -855,16 +906,25 @@ class GameConqueror():
     # read/write data periodically
     def data_worker(self):
         if (not self.is_scanning) and self.command_lock.acquire(0): # non-blocking
+            self.is_data_worker_working = True
             rows = self.get_visible_rows(self.scanresult_tv)
             if rows is not None:
                 (r1, r2) = rows # [r1, r2] rows are visible
                 for i in xrange(r1, r2+1):
                     row = self.scanresult_liststore[i]
-                    addr, cur_value, scanmem_type = row
-                    row[1] = str(self.read_value(addr, TYPENAMES_S2G[scanmem_type.strip()], cur_value))
+                    addr, cur_value, scanmem_type, valid = row
+                    if valid:
+		        new_value = self.read_value(addr, TYPENAMES_S2G[scanmem_type.strip()], cur_value)
+                        if new_value is not None:
+                            row[1] = str(new_value)
+                        else:
+                            row[1] = '??'
+                            row[3] = False
             # write locked values in cheat list and read unlocked values
             for i in xrange(len(self.cheatlist_liststore)):
-                (lockflag, locked, desc, addr, typestr, value) = self.cheatlist_liststore[i]
+                (lockflag, locked, desc, addr, typestr, value, valid) = self.cheatlist_liststore[i]
+                if not valid:
+                    continue
                 if locked:
                     self.write_value(addr, typestr, value)
                 elif i in self.cheatlist_updates:
@@ -872,9 +932,12 @@ class GameConqueror():
                     self.cheatlist_updates.remove(i)
                 else:
                     newvalue = self.read_value(addr, typestr, value)
-                    if newvalue != value and not locked and not self.cheatlist_editing:
-                        self.cheatlist_liststore[i] = (lockflag, locked, desc, addr, typestr, newvalue)
+                    if newvalue is None:
+                        self.cheatlist_liststore[i] = (lockflag, False, desc, addr, typestr, '??', False)
+                    elif newvalue != value and not locked and not self.cheatlist_editing:
+                        self.cheatlist_liststore[i] = (lockflag, locked, desc, addr, typestr, newvalue, valid)
             self.command_lock.release()           
+            self.is_data_worker_working = False 
         return not self.exit_flag
 
     def read_value(self, addr, typestr, prev_value):
@@ -882,7 +945,7 @@ class GameConqueror():
     
     # addr could be int or str
     def read_memory(self, addr, length):
-        if isinstance(addr,int):
+        if not isinstance(addr,str):
             addr = '%x'%(addr,)
         f = tempfile.NamedTemporaryFile()
 
@@ -899,12 +962,13 @@ class GameConqueror():
 #            for byte in bytes: data += chr(int(byte, 16))
         # TODO raise Exception here isn't good
         if len(data) != length:
-            self.show_error('Cannot access target memory')
+            # self.show_error('Cannot access target memory')
+            data = None
         return data
             
     # addr could be int or str
     def write_value(self, addr, typestr, value):
-        if isinstance(addr,int):
+        if not isinstance(addr,str):
             addr = '%x'%(addr,)
 
         self.command_lock.acquire()
