@@ -525,7 +525,7 @@ bool handler__dregion(globals_t * vars, char **argv, unsigned argc)
     unsigned id;
     bool invert = false;
     char *end = NULL, *idstr = NULL, *block = NULL;
-    element_t *np, *p, *pp;
+    element_t *np, *pp;
     list_t *keep = NULL;
     region_t *save;
 
@@ -591,7 +591,7 @@ bool handler__dregion(globals_t * vars, char **argv, unsigned argc)
         
         /* initialise list pointers */
         np = vars->regions->head;
-        p = pp = NULL;
+        pp = NULL;
         
         /* find the correct region node */
         while (np) {
@@ -744,7 +744,7 @@ bool handler__decinc(globals_t * vars, char **argv, unsigned argc)
     }
     else if (strcmp(argv[0], "!=") == 0)
     {
-        m = MATCHCHANGED;
+        m = (argc == 1) ? MATCHCHANGED : MATCHNOTEQUALTO;
     }
     else if (strcmp(argv[0], "<") == 0)
     {
@@ -1296,16 +1296,37 @@ bool handler__dump(globals_t * vars, char **argv, unsigned argc)
             {
                 printf("%02X ", (unsigned char)(buf[buf_idx++]));
             }
+            if(vars->options.dump_with_ascii == 1)
+            {
+                for (j = 0; j < 16; ++j)
+                {
+                    char c = buf[i+j];
+                    printf("%c", isprint(c) ? c : '.');
+                }
+            }
             printf("\n");
         }
         if (i < len)
         {
             if (vars->options.backend == 0)
                 printf("%p: ", addr+i);
-            for (; i < len; ++i)
+            for (j = i; j < len; ++j)
             {
                 printf("%02X ", (unsigned char)(buf[buf_idx++]));
             }
+            if(vars->options.dump_with_ascii == 1)
+            {
+                while(j%16 !=0) // skip "empty" numbers
+                {
+                    printf("   ");
+                    ++j;
+                }
+                for (j = 0; i+j < len; ++j)
+                {
+                    char c = buf[i+j];
+                    printf("%c", isprint(c) ? c : '.');
+                }
+            }
             printf("\n");
         }
     }
@@ -1462,6 +1483,30 @@ bool handler__write(globals_t * vars, char **argv, unsigned argc)
             goto retl;
         }
         int i;
+
+        {
+            // if wildcard is provided in the bytearray, we need the original data.
+            bool wildcard_used = false;
+            for(i = 0; i < data_width; ++i)
+            {
+                if(array[i].is_wildcard == 1)
+                {
+                    wildcard_used = true;
+                    break;
+                }
+            }
+            if (wildcard_used)
+            {
+                if(!read_array(vars->target, addr, buf, data_width))
+                {
+                    show_error("read memory failed.\n");
+                    free(array);
+                    ret = false;
+                    goto retl;
+                }
+            }
+        }
+
         for(i = 0; i < data_width; ++i)
         {
             bytearray_element_t *cur_element = array+i;
@@ -1537,6 +1582,16 @@ bool handler__option(globals_t * vars, char **argv, unsigned argc)
             return false;
         }
     }
+    else if (strcasecmp(argv[1], "dump_with_ascii") == 0)
+    {
+        if (strcmp(argv[2], "0") == 0) {vars->options.dump_with_ascii = 0; }
+        else if (strcmp(argv[2], "1") == 0) {vars->options.dump_with_ascii = 1; }
+        else
+        {
+            show_error("bad value for dump_with_ascii, see `help option`.\n");
+            return false;
+        }
+    }
     else
     {
         show_error("unknown option specified, see `help option`.\n");
