@@ -109,7 +109,7 @@ bool handler__lregions(globals_t * vars, char **argv, unsigned argc);
 
 #define GREATERTHAN_SHRTDOC "match values that have increased or greater than some number"
 #define LESSTHAN_SHRTDOC "match values that have decreased or less than some number"
-#define NOTCHANGED_SHRTDOC "match all variables that have not changed since last scan"
+#define NOTCHANGED_SHRTDOC "match all variables that have changed since last scan"
 #define CHANGED_SHRTDOC "match all variables that have not changed since last scan"
 #define INCREASED_SHRTDOC "match values that have increased by some given number"
 #define DECREASED_SHRTDOC "match values that have decreased by some given number"
@@ -285,6 +285,13 @@ bool handler__write(globals_t * vars, char **argv, unsigned argc);
                  "\t0:\tdisabled\n" \
                  "\t1:\tenabled\n" \
                  "\n" \
+                 "dump_with_ascii\twhether to print ascii characters with a memory dump\n" \
+                 "\t\t\tDefault:1\n" \
+                 "\n" \
+                 "\tpossible values:\n" \
+                 "\t0:\tdisabled\n" \
+                 "\t1:\tenabled\n" \
+                 "\n" \
                  "Example:\n" \
                  "\toption scan_data_type int32\n"
 
